//@author: vux
//@help: template for simple compute shader (buffer write)
//@tags: template
//@credits: 



struct MeshPoint
{
	float3 pos; 
	float3 normal;
	float4 tangent;
	float2 uv;
	
	int3 tanInt1;
	int3 tanInt2;
	int3 normalInt;
};

static float PI = 3.14159265359;

//Buffers
RWStructuredBuffer <MeshPoint> meshVBuff;
RWStructuredBuffer <uint> trianglesBuff;
RWStructuredBuffer <float3> meshNormalsBuff;
RWStructuredBuffer <float3> idBuff;

//SuperFormula params
float pi_loops = 1.0;
float sf_M_1 = 3.0;
float sf_N1_1 = 0.5;
float sf_N2_1 = 0.5;
float sf_N3_1 = 0.5;
float sf_M_2 = 0.5;
float sf_N1_2 = 0.5;
float sf_N2_2 = 0.5;
float sf_N3_2 = 0.5;
float a1 = 1, b1 = 1, a2 = 1, b2 = 1;
float sf_scale = 1.0;

//resolution
uint xPoints = 100;
uint yPoints = 100;
//int numIndices = 1;

//non riesco a capire perch % e mod mi restituiscono risultati sbagliati.
uint myModuloOp(uint a, uint b)
{
	return a - (b * floor(a / b));
}

float3 safeNormalize(float3 v)
{
	if(length(v)>0)
	{
		return normalize(v);
	}
	return float3(0,1,0);
}

float CalculateSuperShapeRadius(float m, float n1,float n2,float n3,float angle, float a, float b)
{
	float r;
	float t1,t2;

	t1 = cos(m * angle / 4) / a;
	t1 = abs(t1);
	t1 = pow(t1, n2);

	t2 = sin(m * angle / 4) / b;
	t2 = abs(t2);
	t2 = pow(t2, n3);

	r = pow(t1 + t2, 1 / n1);
	//r = (r != 0) ? (1.0 / r) : r;
	r = 1 / r;
	return r;
}

float3 SuperFormula3D(uint i, uint j)
{ 
	//SUPERFORMULA
	float theta = PI * (float(i) / (xPoints));
	float r2 = CalculateSuperShapeRadius(sf_M_1, sf_N1_1, sf_N2_1, sf_N3_1, theta, a1, b1) ;
	
	float phi =  (2.0 * PI) * (float(j) / (yPoints-1));
	float r1 = CalculateSuperShapeRadius(sf_M_2, sf_N1_2, sf_N2_2, sf_N3_2, phi, a2, b2) ;

	//MAP TO SPHERE
	float x =  r1 * cos(phi) * r2 * sin(theta);
	float y =  r1 * sin(phi) * r2 * sin(theta);
	float z =  r2 * cos(theta);

	float3 outPos = float3(x,z,y) * sf_scale;
	return outPos;
}


[numthreads(32,1,1)]
void CalculateSuperFormula3D (uint3 id : SV_DispatchThreadID)
{
	//GET ID
	uint cID = id.x;
	
	if(id.x > xPoints * yPoints) return;

	uint i = uint((float)cID / xPoints);
	uint j = myModuloOp(cID, yPoints);

	float3 sfNewPos = SuperFormula3D(i, j);
	meshVBuff[cID].pos = sfNewPos;

	float3 d = sfNewPos;
    float u = (j >= yPoints-1) ? 1 : (1 - (0.5 + (atan2(d.z, -d.x) / (2 * PI) )));
    float v = 0.5 + (sin(d.y) / PI );

	meshVBuff[cID].uv = clamp(float2(u,v),0,1);
	meshVBuff[cID].normalInt = int3(0,0,0); //reset normal and tangents for accumulation
	meshVBuff[cID].tanInt1 = int3(0,0,0);
	meshVBuff[cID].tanInt2 = int3(0,0,0);
	
	
	//DEBUG BUFFER
	//idBuff[cID] = float3(cID, i, j) ;
}
technique11 Apply
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CalculateSuperFormula3D() ) );
	}
}




